#!/usr/bin/python
# -*- coding: utf-8 -*-
# nautilus-pastebin - Nautilus extension to paste a file to a pastebin service
# Written by:
#    Alessio Treglia <quadrispro@ubuntu.com>
# Copyright (C) 2009, Alessio Treglia
#
# Part of this code has been taken from pastebinit script
# pastebinit was written by: 
#    Stéphane Graber <stgraber@stgraber.org>
#    Daniel Bartlett <dan@f-box.org>
#    David Paleino <d.paleino@gmail.com>
# URL: http://www.stgraber.org/download/projects/pastebin/
# Copyright (C) 2006-2009, Stéphane Graber, Daniel Bartlett, David Paleino
# Distributed under the terms of the GNU GPL-2 license
#
# This package is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This package is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this package; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
#

import urllib
import os, re
import mimetypes

from ConfigParser import ConfigParser

# Custom urlopener to handle 401's
class pasteURLopener(urllib.FancyURLopener):
    """Do the HTTP requests."""
    def http_error_401(self, url, fp, errcode, errmsg, headers, data=None):
            return None

class PastebinConfiguration(object):
    PARAMETERS = ["website",
        "validator",
        "content",
        "user",
        "jabberid",
        "version",
        "format",
        "parentid",
        "permatag",
        "title",
        "username",
        "password",
        "default_mime"]

    def __setDefaultParameters(self):
        for k in PastebinConfiguration.PARAMETERS:
            self.__dict__.setdefault(k,'')
            if k in self._optionsDict.keys():
                setattr(self, k, self._optionsDict[k])

    def __setGlobalParameters(self):
        for k in self._globalsDict.keys():
            setattr(self, k, self._globalsDict[k])

    def __getMappingByName(self, optname):
        map = {}
        if optname in self._optionsDict:
            exec(self._optionsDict[optname])
        return map

    def __init__(self, globalsDict, optionsDict):
        self._optionsDict = optionsDict
        self._globalsDict = globalsDict
        self.__setGlobalParameters()
        self.__setDefaultParameters()
        self.aliases = self.__getMappingByName('aliases')
        self.extras = self.__getMappingByName('extras')
        self.mimetypes = self.__getMappingByName('mimetypes')
        self.mimetypes.setdefault('text/plain', self._optionsDict['default_mime'])

    def setFormatByMimeType(self, mimetype):
        if mimetype in self.mimetypes.keys():
            self.format = self.mimetypes[mimetype]
        elif mimetype.split('/')[0] == 'text':
            self.format = self.mimetypes['text/plain']
        else:
            return False
        return True
    
    def __str__(self):
        s = 'Configuration:\n'
        s += "mimetypes = %s\n" % self.mimetypes.__str__()
        s += "aliases = %s" % self.aliases.__str__()
        s += "extras = %s" % self.extras.__str__()
        return s

    def setContent(self, pastefile):
        fp = open(pastefile)
        content = fp.read()
        fp.close()
        if not content:
            return False
        # OK, let's go...
        self.content = content
        self.title = os.path.split(pastefile)[-1]
        return True

class PastebinValidatorFactory(object):
    """Singleton factory which returns a validator object dinamically."""
    __instance = None
    class __impl(object):
        """Implementation of the class' methods"""
        def getValidator(self, pasteconf):
            validator = pasteconf.validator
            obj = None
            try:
                exec("obj = %sValidator(pasteconf)" % validator)
            except Exception, e:
                pass
            return obj

    def __getattr__(self, attr):
        return getattr(self.__instance, attr)
    def __setattr__(self, attr, value):
        return setattr(self.__instance, attr, value)

    def __init__(self):
        if PastebinValidatorFactory.__instance is None:
            PastebinValidatorFactory.__instance = PastebinValidatorFactory.__impl()
        self.__dict__['_PastebinValidatorFactory__instance'] = PastebinValidatorFactory.__instance

class GenericPastebinValidator(object):
    NOT_EMPTY_FIELDS = ['website', 'content', 'format', 'title']

    def checkAliases(self):
        return all(
            map(
                lambda x: x in self.conf.__dict__, self.conf.aliases.keys())
                )

    def checkExtras(self): return True

    def __init__(self, pastebinConfiguration):
        """Class constructor"""
        try:
            toAdd = getattr(cls, 'notEmptyFields')
            GenericPastebinValidator.NOT_EMPTY_FIELDS.extend(toAdd)
        except:
            pass
        self.conf = pastebinConfiguration

    def validate(self):
        """Do the validation of the configuration file."""
        if self.conf.user == '':
            self.conf.user = os.environ.get('USER')
        return self.checkNotEmpty() and self.checkAliases() and self.checkExtras()

    def checkNotEmpty(self):
        """Verify whether all the opts' keys are non-empty."""
        return all(map(lambda x: getattr(self.conf, x) != '', GenericPastebinValidator.NOT_EMPTY_FIELDS))

    def getParameters(self):
        params = {}
        # Set the aliases
        for k in self.conf.aliases.keys():
            for v in self.conf.aliases[k]:
                params[v] = getattr(self.conf, k)
        for k in self.conf.extras.keys():
            params[k] = self.conf.extras[k]
        return params
    
    def updateParametersAfterSending(self): pass

class PasteyNetValidator(GenericPastebinValidator):

    def parentFixup(self, website, parentid):
        if parentid == "":
            return ""
        url_opener = pasteURLopener()
        page = url_opener.open(website + '/' + parentid, None)
        matches = re.split('<input.*?name="parent".*?value="(.*?)"', page.read())
        if len(matches) <= 1 or re.match(parentid, matches[1]) == None:
            # The obfuscated version didn't begin with the partial version,
            # or unable to find the obfuscated version for some reason!
            # Create a paste with no parent (should we throw, instead?)
            return ""
        return matches[1]

    def getParameters(self):
        params = GenericPastebinValidator.getParameters(self)
        params['parent'] = self.parentFixup(self.conf.website, self.conf.parentid)
        return params

class Paster(object):
    def __init__(self, pasteconf):
        self.pasteconf = pasteconf

    def paste(self):
        validator = PastebinValidatorFactory().getValidator(self.pasteconf)
        pasteurl = None
        
        if (not validator) or (not validator.validate()):
            return pasteurl

        website = self.pasteconf.website
        params = validator.getParameters()

        if not re.search(".*/", website):
            website += "/"

        reLink=None
        tmp_page=""
        if params.__contains__("page"):
            website+=params['page']
            tmp_page=params['page']
            params.__delitem__("page")
        if params.__contains__("regexp"):
            reLink=params['regexp']
            params.__delitem__("regexp")
        params = urllib.urlencode(params) #Convert to a format usable with the HTML POST

        try:
            url_opener = pasteURLopener()
            page = url_opener.open(website, params) #Send the informations and be redirected to the final page
            summary = 'File pasted to: '
            URLmarkup = '<a href="%(pasteurl)s">%(pasteurl)s</a>'
            if reLink: #Check if we have to apply a regexp
                if website.count(tmp_page) > 1:
                    website = website.rstrip(tmp_page)
                else:
                    website = website.replace(tmp_page, "")
                pasteurl = website + "/" + re.split(reLink, page.read())[1]
            else:
                pasteurl = page.url
            message = URLmarkup % { 'pasteurl': pasteurl }

        except Exception, value:
            pasteurl = None
        return pasteurl

